@@ -83,6 +83,65 @@ def _speech_collate_fn(batch, pad_id):
     return audio_signal, audio_lengths, tokens, tokens_lengths
 
 
+class ASRManifestProcessor:
+    """
+    Class that processes a manifest json file containing paths to audio files, transcripts, and durations (in seconds).
+    Each new line is a different sample. Example below:
+    {"audio_filepath": "/path/to/audio.wav", "text_filepath": "/path/to/audio.txt", "duration": 23.147}
+    ...
+    {"audio_filepath": "/path/to/audio.wav", "text": "the transcription", "offset": 301.75, "duration": 0.82, "utt":
+    "utterance_id", "ctm_utt": "en_4156", "side": "A"}
+    Args:
+        manifest_filepath: Path to manifest json as described above. Can be comma-separated paths.
+        parser: Str for a language specific preprocessor or a callable.
+        max_duration: If audio exceeds this length, do not include in dataset.
+        min_duration: If audio is less than this length, do not include in dataset.
+        max_utts: Limit number of utterances.
+        bos_id: Id of beginning of sequence symbol to append if not None.
+        eos_id: Id of end of sequence symbol to append if not None.
+        pad_id: Id of pad symbol. Defaults to 0.
+    """
+
+    def __init__(
+        self,
+        manifest_filepath: str,
+        parser: Union[str, Callable],
+        max_duration: Optional[float] = None,
+        min_duration: Optional[float] = None,
+        max_utts: int = 0,
+        bos_id: Optional[int] = None,
+        eos_id: Optional[int] = None,
+        pad_id: int = 0,
+    ):
+        self.parser = parser
+
+        self.collection = collections.ASRAudioText(
+            manifests_files=manifest_filepath.split(','),
+            parser=parser,
+            min_duration=min_duration,
+            max_duration=max_duration,
+            max_number=max_utts,
+        )
+
+        self.eos_id = eos_id
+        self.bos_id = bos_id
+        self.pad_id = pad_id
+
+    def process_text(self, index) -> (List[int], int):
+        sample = self.collection[index]
+
+        t, tl = sample.text_tokens, len(sample.text_tokens)
+
+        if self.bos_id is not None:
+            t = [self.bos_id] + t
+            tl += 1
+        if self.eos_id is not None:
+            t = t + [self.eos_id]
+            tl += 1
+
+        return t, tl
+
+
 class _AudioTextDataset(Dataset):
     """
     Dataset that loads tensors via a json file containing paths to audio files, transcripts, and durations (in seconds).
@@ -134,24 +193,21 @@ class _AudioTextDataset(Dataset):
         eos_id: Optional[int] = None,
         pad_id: int = 0,
     ):
-        self.parser = parser
-
-        self.collection = collections.ASRAudioText(
-            manifests_files=manifest_filepath.split(','),
+        self.manifest_processor = ASRManifestProcessor(
+            manifest_filepath=manifest_filepath,
             parser=parser,
-            min_duration=min_duration,
             max_duration=max_duration,
-            max_number=max_utts,
+            min_duration=min_duration,
+            max_utts=max_utts,
+            bos_id=bos_id,
+            eos_id=eos_id,
+            pad_id=pad_id,
         )
-
         self.featurizer = WaveformFeaturizer(sample_rate=sample_rate, int_values=int_values, augmentor=augmentor)
         self.trim = trim
-        self.eos_id = eos_id
-        self.bos_id = bos_id
-        self.pad_id = pad_id
 
     def __getitem__(self, index):
-        sample = self.collection[index]
+        sample = self.manifest_processor.collection[index]
         offset = sample.offset
 
         if offset is None:
@@ -162,23 +218,17 @@ class _AudioTextDataset(Dataset):
         )
         f, fl = features, torch.tensor(features.shape[0]).long()
 
-        t, tl = sample.text_tokens, len(sample.text_tokens)
-        if self.bos_id is not None:
-            t = [self.bos_id] + t
-            tl += 1
-        if self.eos_id is not None:
-            t = t + [self.eos_id]
-            tl += 1
+        t, tl = self.manifest_processor.process_text(index)
 
         output = f, fl, torch.tensor(t).long(), torch.tensor(tl).long()
 
         return output
 
     def __len__(self):
-        return len(self.collection)
+        return len(self.manifest_processor.collection)
 
     def _collate_fn(self, batch):
-        return _speech_collate_fn(batch, pad_id=self.pad_id)
+        return _speech_collate_fn(batch, pad_id=self.manifest_processor.pad_id)
 
 
 class AudioToCharDataset(_AudioTextDataset):
@@ -375,7 +425,7 @@ class AudioToCharWithDursF0Dataset(AudioToCharDataset):
 
         tags = []
         self.id2enc_text = {}
-        for i, e in enumerate(self.collection):
+        for i, e in enumerate(self.manifest_processor.collection):
             tag = os.path.splitext(os.path.basename(e.audio_file))[0]
             tags.append(tag)
             # cache vocab encoding
@@ -533,7 +583,7 @@ class AudioToCharWithPriorDataset(AudioToCharDataset):
 
         self.n_window_stride = n_window_stride
         self.id2enc_text = {}
-        for i, e in enumerate(self.collection):
+        for i, e in enumerate(self.manifest_processor.collection):
             # cache vocab encoding
             self.id2enc_text[i] = self.vocab.encode(e.text_raw)
 
@@ -615,7 +665,7 @@ class AudioToCharWithPriorAndPitchDataset(AudioToCharWithPriorDataset):
 
     def __getitem__(self, item):
         audio, audio_len, text, text_len, attn_prior = super().__getitem__(item)
-        tag = Path(self.collection[item].audio_file).stem
+        tag = Path(self.manifest_processor.collection[item].audio_file).stem
         pitch_path = (
             Path(self.attn_prior_folder)
             / f"{tag}_pitch_pyin_fmin{self.pitch_fmin}_fmax{self.pitch_fmax}_fl{self.n_window_size}.npy"
@@ -674,18 +724,19 @@ class FastPitchDataset(_AudioTextDataset):
     def __getitem__(self, item):
         audio, audio_len, text, text_len = super().__getitem__(item)  # noqa
 
-        audio_path = self.collection[item].audio_file
+        audio_path = self.manifest_processor.collection[item].audio_file
         durs_path = audio_path.replace('/wavs/', '/fastpitch/durations/').replace('.wav', '.pt')
         pitch_path = audio_path.replace('/wavs/', '/fastpitch/pitch_char/').replace('.wav', '.pt')
         speaker = None
-        if self.collection[item].speaker is not None:
-            speaker = torch.zeros_like(text_len).fill_(self.collection[item].speaker)
+        if self.manifest_processor.collection[item].speaker is not None:
+            speaker = torch.zeros_like(text_len).fill_(self.manifest_processor.collection[item].speaker)
 
         return (audio, audio_len, text, text_len, torch.load(durs_path), torch.load(pitch_path), speaker, audio_path)
 
     def _collate_fn(self, batch):
+        pad_id = self.manifest_processor.pad_id
         asr_batch = list(zip(*batch))[:4]
-        asr_batch = _speech_collate_fn(list(zip(*asr_batch)), pad_id=self.pad_id)
+        asr_batch = _speech_collate_fn(list(zip(*asr_batch)), pad_id=pad_id)
         audio, audio_len, text, text_len = asr_batch
 
         max_tokens_len = text.size(1)
@@ -694,8 +745,8 @@ class FastPitchDataset(_AudioTextDataset):
             pad = (0, max_tokens_len - tokens_i_len.item())
             assert len(durs_i) == len(pitch_i), f"{len(durs_i)}, {len(pitch_i)}: {path_i}"
             assert len(durs_i) == tokens_i_len, f"{len(durs_i)}, {tokens_i_len}:  {path_i}"
-            durs.append(F.pad(durs_i, pad, value=self.pad_id))
-            pitch.append(F.pad(pitch_i, pad, value=self.pad_id))
+            durs.append(F.pad(durs_i, pad, value=pad_id))
+            pitch.append(F.pad(pitch_i, pad, value=pad_id))
             speakers.append(speaker_i)
 
         return (
@@ -1249,6 +1300,8 @@ class TarredAudioToBPEDataset(_TarredAudioToTextDataset):
         trim (bool): Whether to use trim silence from beginning and end
             of audio signal using librosa.effects.trim().
             Defaults to False.
+        use_start_end_token: Boolean which dictates whether to add [BOS] and [EOS]
+            tokens to beginning and ending of speech respectively.
         pad_id (id): Token used to pad when collating samples in batches.
             If this is None, pads using 0s.
             Defaults to None.
