@@ -24,6 +24,8 @@ from nemo.collections.asr.modules.conv_asr import (
     ConvASRDecoder,
     ConvASRDecoderClassification,
     ConvASREncoder,
+    ECAPAEncoder,
+    ParallelConvASREncoder,
     SpeakerDecoder,
 )
 from nemo.collections.asr.modules.lstm_decoder import LSTMDecoder
