@@ -17,6 +17,7 @@ from typing import Callable, List, Optional, Tuple
 
 import torch
 import torch.nn as nn
+import torch.nn.functional as F
 from torch import Tensor
 from torch.nn.init import _calculate_correct_fan
 from torch.nn.modules.utils import _single
@@ -43,7 +44,7 @@ def tds_uniform_(tensor, mode='fan_in'):
     Normalized to -
 
     .. math::
-        \text{bound} = \text{2} \times \sqrt{\frac{1}{\text{fan\_mode}}}
+        \\text{bound} = \\text{2} \\times \\sqrt{\\frac{1}{\\text{fan\\_mode}}}
 
     Args:
         tensor: an n-dimensional `torch.Tensor`
@@ -66,7 +67,7 @@ def tds_normal_(tensor, mode='fan_in'):
     Normalized to -
 
     .. math::
-        \text{bound} = \text{2} \times \sqrt{\frac{1}{\text{fan\_mode}}}
+        \\text{bound} = \\text{2} \\times \\sqrt{\\frac{1}{\\text{fan\\_mode}}}
 
     Args:
         tensor: an n-dimensional `torch.Tensor`
@@ -162,6 +163,27 @@ def get_asymtric_padding(kernel_size, stride, dilation, future_context):
     return (left_context, right_context)
 
 
+@torch.jit.script
+def _se_pool_step_script(x, context_window: int):
+    timesteps = x.shape[-1]
+    if timesteps < context_window:
+        y = F.adaptive_avg_pool1d(x, 1)
+    else:
+        y = F.avg_pool1d(x, context_window, 1)  # [B, C, T - context_window + 1]
+    return y
+
+
+@torch.jit.script
+def _masked_conv_init_lens(lens: torch.Tensor, current_maxlen: int, original_maxlen: torch.Tensor):
+    if current_maxlen > original_maxlen:
+        new_lens = torch.arange(current_maxlen)
+        new_max_lens = torch.tensor(current_maxlen)
+    else:
+        new_lens = lens
+        new_max_lens = original_maxlen
+    return new_lens, new_max_lens
+
+
 class MaskedConv1d(nn.Module):
     __constants__ = ["use_conv_mask", "real_out_channels", "heads"]
 
@@ -245,8 +267,8 @@ class MaskedConv1d(nn.Module):
         # `self.lens` caches consecutive integers from 0 to `self.max_len` that are used to compute the mask for a
         # batch. Recomputed to bigger size as needed. Stored on a device of the latest batch lens.
         if self.use_mask:
-            self.max_len = 0
-            self.lens = None
+            self.max_len = torch.tensor(0)
+            self.lens = torch.tensor(0)
 
     def get_seq_len(self, lens):
         if self.same_padding or self.same_padding_asymmetric:
@@ -263,15 +285,7 @@ class MaskedConv1d(nn.Module):
 
     def forward(self, x, lens):
         if self.use_mask:
-            max_len = x.size(2)
-            if max_len > self.max_len:
-                self.lens = torch.arange(max_len)
-                self.max_len = max_len
-
-            self.lens = self.lens.to(lens.device)
-            mask = self.lens[:max_len].unsqueeze(0) < lens.unsqueeze(1)
-            x = x * mask.unsqueeze(1).to(device=x.device)
-            lens = self.get_seq_len(lens)
+            x, lens = self.update_masked_length(x, lens)
 
         # asymmtric pad if necessary
         if self.pad_layer is not None:
@@ -288,6 +302,15 @@ class MaskedConv1d(nn.Module):
 
         return out, lens
 
+    def update_masked_length(self, x, lens):
+        max_len = x.size(2)
+        self.lens, self.max_len = _masked_conv_init_lens(self.lens, max_len, self.max_len)
+        self.lens = self.lens.to(lens.device)
+        mask = self.lens[:max_len].unsqueeze(0) < lens.unsqueeze(1)
+        x = x * mask.unsqueeze(1).to(device=x.device)
+        lens = self.get_seq_len(lens)
+        return x, lens
+
 
 class GroupShuffle(nn.Module):
     def __init__(self, groups, channels):
@@ -365,24 +388,26 @@ class SqueezeExcite(nn.Module):
 
     def forward(self, x):
         # The use of negative indices on the transpose allow for expanded SqueezeExcite
-        batch, channels, timesteps = x.size()[:3]
         # Computes in float32 to avoid instabilities during training with AMP.
         with torch.cuda.amp.autocast(enabled=False):
             x = x.float()
-            if timesteps < self.context_window:
-                y = self.gap(x)
-            else:
-                y = self.pool(x)  # [B, C, T - context_window + 1]
+            y = self._se_pool_step(x)
             y = y.transpose(1, -1)  # [B, T - context_window + 1, C]
             y = self.fc(y)  # [B, T - context_window + 1, C]
             y = y.transpose(1, -1)  # [B, C, T - context_window + 1]
-
-        if self.context_window > 0:
-            y = torch.nn.functional.interpolate(y, size=timesteps, mode=self.interpolation_mode)
+        if self.context_window >= 0:
+            y = F.interpolate(y, size=x.shape[-1], mode=self.interpolation_mode)
 
         y = torch.sigmoid(y)
         return x * y
 
+    def _se_pool_step(self, x):
+        if self.context_window < 0:
+            y = self.pool(x)
+        else:
+            y = _se_pool_step_script(x, self.context_window)
+        return y
+
     def change_context_window(self, context_window: int):
         """
         Update the context window of the SqueezeExcitation module, in-place if possible.
@@ -404,9 +429,9 @@ class SqueezeExcite(nn.Module):
         if hasattr(self, 'context_window'):
             logging.info(f"Changing Squeeze-Excitation context window from {self.context_window} to {context_window}")
 
-        self.context_window = int(context_window)
+        self.context_window = context_window
 
-        if self.context_window <= 0:
+        if self.context_window < 0:
             if PYTORCH_QUANTIZATION_AVAILABLE and self._quantize:
                 if not isinstance(self.pool, quant_nn.QuantAdaptiveAvgPool1d(1)):
                     self.pool = quant_nn.QuantAdaptiveAvgPool1d(1)  # context window = T
