@@ -211,8 +211,28 @@ class MTEncDecModel(EncDecNLPModel):
         ids[ids >= self.decoder_tokenizer.vocab_size] = self.decoder_tokenizer.unk_id
         return ids
 
+    def test_encoder_ids(self, ids, raise_error=False):
+        invalid_ids = (ids >= self.encoder_tokenizer.vocab_size).any()
+
+        if raise_error and invalid_ids:
+            raise ValueError("Encoder ids are out of range (tip: check encoder tokenizer)")
+
+        return not invalid_ids
+
+    def test_decoder_ids(self, ids, raise_error=False):
+        invalid_ids = (ids >= self.decoder_tokenizer.vocab_size).any()
+
+        if raise_error and invalid_ids:
+            raise ValueError("Decoder ids are out of range (tip: check decoder tokenizer)")
+
+        return not invalid_ids
+
     @typecheck()
     def forward(self, src, src_mask, tgt, tgt_mask):
+        # test src/tgt for id range (i.e., hellp in catching wrong tokenizer)
+        self.test_encoder_ids(src, raise_error=True)
+        self.test_decoder_ids(tgt, raise_error=True)
+
         src_hiddens = self.encoder(input_ids=src, encoder_mask=src_mask)
         tgt_hiddens = self.decoder(
             input_ids=tgt, decoder_mask=tgt_mask, encoder_embeddings=src_hiddens, encoder_mask=src_mask
@@ -564,10 +584,11 @@ class MTEncDecModel(EncDecNLPModel):
             sampler = pt_data.RandomSampler(dataset)
         else:
             sampler = pt_data.SequentialSampler(dataset)
+
         return torch.utils.data.DataLoader(
             dataset=dataset,
             batch_size=1,
-            sampler=None if cfg.get("use_tarred_dataset", False) else sampler,
+            sampler=None if (cfg.get("use_tarred_dataset", False) or isinstance(dataset, ConcatDataset)) else sampler,
             num_workers=cfg.get("num_workers", 2),
             pin_memory=cfg.get("pin_memory", False),
             drop_last=cfg.get("drop_last", False),
