@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import glob
 import hashlib
 import json
 import os
@@ -38,8 +37,8 @@ from nemo.collections.nlp.modules.common.tokenizer_utils import get_tokenizer
 from nemo.collections.nlp.parts.nlp_overrides import NLPCheckpointConnector
 from nemo.core.classes import ModelPT
 from nemo.core.classes.exportable import Exportable
+from nemo.core.connectors.save_restore_connector import SaveRestoreConnector
 from nemo.utils import AppState, logging
-from nemo.utils.exp_manager import configure_checkpointing
 from nemo.utils.get_rank import is_global_rank_zero
 
 __all__ = ['NLPModel']
@@ -57,7 +56,9 @@ class NLPModel(ModelPT, Exportable):
         super().__init__(cfg, trainer)
         self.set_world_size(trainer)
 
-    def register_artifact(self, config_path: str, src: str, verify_src_exists: bool = False):
+    def register_artifact(
+        self, config_path: str, src: str, verify_src_exists: bool = False,
+    ):
         """ Overrides ModelPT register_artifact default behavior. NLP models usually need artifacts that are optional."""
         return super().register_artifact(config_path, src, verify_src_exists=verify_src_exists)
 
@@ -335,6 +336,7 @@ class NLPModel(ModelPT, Exportable):
         strict: bool = True,
         return_config: bool = False,
         trainer: Trainer = None,
+        save_restore_connector: SaveRestoreConnector = None,
     ):
         """
         Restores model instance (weights and configuration) from .nemo file.
@@ -359,6 +361,9 @@ class NLPModel(ModelPT, Exportable):
         Returns:
             An instance of type cls or its underlying config (if return_config is set).
         """
+        if save_restore_connector is None:
+            save_restore_connector = SaveRestoreConnector()
+
         if not os.path.exists(restore_path):
             raise FileNotFoundError(f"Can't find {restore_path}")
 
@@ -420,13 +425,21 @@ class NLPModel(ModelPT, Exportable):
             model_parallel_rank = compute_model_parallel_rank(trainer.local_rank, app_state.model_parallel_size)
             app_state.model_parallel_rank = model_parallel_rank
 
-            restored_model = cls._default_restore_from(
-                restore_path, override_config_path, map_location, strict, return_config
+            cls.update_save_restore_connector(save_restore_connector)
+            restored_model = cls._save_restore_connector.restore_from(
+                cls, app_state.model_restore_path, override_config_path, map_location, strict, return_config
             )
             restored_model.set_trainer(trainer)
             return restored_model
         else:
-            return super().restore_from(restore_path, override_config_path, map_location, strict, return_config)
+            return super().restore_from(
+                app_state.model_restore_path,
+                override_config_path,
+                map_location,
+                strict,
+                return_config,
+                save_restore_connector=save_restore_connector,
+            )
 
     @rank_zero_only
     def register_megatron_checkpoint_version(self):
