@@ -29,6 +29,7 @@ import wrapt
 from omegaconf import DictConfig, OmegaConf
 
 import nemo
+from nemo.core.connectors.save_restore_connector import SaveRestoreConnector
 from nemo.core.neural_types import NeuralType, NeuralTypeComparisonResult
 from nemo.utils import logging
 from nemo.utils.cloud import maybe_download_from_cloud
@@ -622,6 +623,7 @@ class Model(Typing, Serialization, FileIO):
         map_location: Optional['torch.device'] = None,
         strict: bool = True,
         return_config: bool = False,
+        save_restore_connector: SaveRestoreConnector = None,
     ):
         """
         Instantiates an instance of NeMo from NVIDIA NGC cloud
@@ -641,6 +643,9 @@ class Model(Typing, Serialization, FileIO):
         Returns:
             A model instance of a particular model class or its underlying config (if return_config is set).
         """
+        if save_restore_connector is None:
+            save_restore_connector = SaveRestoreConnector()
+
         location_in_the_cloud = None
         description = None
         models = cls.list_available_models()
@@ -682,6 +687,7 @@ class Model(Typing, Serialization, FileIO):
             map_location=map_location,
             strict=strict,
             return_config=return_config,
+            save_restore_connector=save_restore_connector,
         )
         return instance
 
