@@ -30,6 +30,7 @@ from omegaconf import DictConfig, OmegaConf, open_dict
 from pytorch_lightning.callbacks import ModelCheckpoint
 from pytorch_lightning.loggers import LoggerCollection as _LoggerCollection
 from pytorch_lightning.loggers import TensorBoardLogger, WandbLogger
+from pytorch_lightning.plugins.training_type.ddp import DDPPlugin
 from pytorch_lightning.utilities import rank_zero_only
 from pytorch_lightning.utilities.types import _METRIC
 
@@ -132,7 +133,7 @@ def exp_manager(trainer: 'pytorch_lightning.Trainer', cfg: Optional[Union[DictCo
                 lightning's TensorboardLogger system of using version_{int}.
             - use_datetime_version (bool): Whether to use a datetime string for version. Defaults to True.
             - resume_if_exists (bool): Whether this experiment is resuming from a previous run. If True, it sets
-                trainer.resume_from_checkpoint so that the trainer should auto-resume. exp_manager will move files
+                trainer.checkpoint_connector.resume_checkpoint_path so that the trainer should auto-resume. exp_manager will move files
                 under log_dir to log_dir/run_{int}. Defaults to False. From v1.0.0, when resume_if_exists is True,
                 we would not create version folders to make it easier to find the log folder for next runs.
             - resume_past_end (bool): exp_manager errors out if resume_if_exists is True and a checkpoint matching
@@ -292,7 +293,7 @@ def error_checks(trainer: 'pytorch_lightning.Trainer', cfg: Optional[Union[DictC
             "You are running multi-node training without SLURM handling the processes."
             " Please note that this is not tested in NeMo and could result in errors."
         )
-    if trainer.num_gpus > 1 and not trainer.use_ddp:
+    if trainer.num_gpus > 1 and not isinstance(trainer.accelerator.training_type_plugin, DDPPlugin):
         logging.error(
             "You are running multi-gpu without ddp.Please note that this is not tested in NeMo and could result in "
             "errors."
@@ -306,7 +307,7 @@ def check_resume(
     resume_ignore_no_checkpoint: bool = False,
 ):
     """Checks that resume=True was used correctly with the arguments pass to exp_manager. Sets
-    trainer.resume_from_checkpoint as necessary.
+    trainer.checkpoint_connector.resume_checkpoint_path as necessary.
 
     Returns:
         log_dir (Path): the log_dir
@@ -361,7 +362,7 @@ def check_resume(
         logging.info(f"Resuming from {last_checkpoints[0]}")
         checkpoint = last_checkpoints[0]
 
-    trainer.resume_from_checkpoint = str(checkpoint)
+    trainer.checkpoint_connector.resume_checkpoint_path = str(checkpoint)
 
     if is_global_rank_zero():
         # Check to see if any files exist that need to be moved
@@ -696,10 +697,10 @@ class NeMoModelCheckpoint(ModelCheckpoint):
         # TODO: make this work for model parallel, need to call on data parallel rank 0 and update best_model_path
         # Load the best model and then re-save it
         if self.save_best_model:
-            trainer.checkpoint_connector.restore(self.best_model_path, on_gpu=trainer.on_gpu)
+            trainer.checkpoint_connector.restore(self.best_model_path)
         pl_module.save_to(save_path=os.path.join(self.dirpath, self.prefix + self.postfix))
 
-    def _del_model(self, filepath: str) -> None:
+    def _del_model(self, trainer: "pl.Trainer", filepath: str) -> None:
         """ Overrides PTL method to account for model parallel checkpoints.
             Updates checkpoint path based on model parallel rank.
         """
@@ -712,12 +713,11 @@ class NeMoModelCheckpoint(ModelCheckpoint):
 
             # each model parallel rank needs to remove its model
             if app_state.data_parallel_rank == 0:
-                if self._fs.exists(filepath):
-                    self._fs.rm(filepath)
-                    logging.info(f"Removed model parallel checkpoint: {filepath}")
+                super()._del_model(trainer, filepath)
+                logging.info(f"Removed model parallel checkpoint: {filepath}")
 
         else:
-            return super()._del_model(filepath)
+            return super()._del_model(trainer, filepath)
 
     def _save_last_checkpoint(self, trainer: 'pl.Trainer', monitor_candidates: Dict[str, _METRIC]) -> None:
         """ Overrides PTL method to account for model parallel checkpoints.
@@ -834,7 +834,7 @@ def configure_checkpointing(
         params.every_n_val_epochs = params.period
 
     checkpoint_callback = NeMoModelCheckpoint(n_resume=resume, **params)
-    checkpoint_callback.last_model_path = trainer.resume_from_checkpoint or ""
+    checkpoint_callback.last_model_path = trainer.checkpoint_connector.resume_checkpoint_path or ""
     trainer.callbacks.append(checkpoint_callback)
 
 
