@@ -53,6 +53,12 @@ def pytest_addoption(parser):
         help="numba compatibility checks will be relaxed to just availability of cuda, "
         "without cuda compatibility matrix check",
     )
+    parser.addoption(
+        '--tn_cache_dir',
+        type=str,
+        default=None,
+        help="path to a directory with .far grammars for CPU TN/ITN tests, (DEFAULT: None, i.e. no cache)",
+    )
 
 
 @pytest.fixture
@@ -186,3 +192,8 @@ def pytest_configure(config):
 
         print("Setting numba compat :", config.option.relax_numba_compat)
         numba_utils.set_numba_compat_strictness(strict=config.option.relax_numba_compat)
+
+    # Set cache directory for TN/ITN tests
+    from .nemo_text_processing.utils import set_cache_dir
+
+    set_cache_dir(config.option.tn_cache_dir)
